<?php

namespace Drupal\language_negotiation_matrix\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\language_negotiation_matrix\Plugin\LanguageNegotiation\LanguageNegotiationUrlMatrix;

/**
 * Configure the URL language negotiation method for this site.
 *
 * @internal
 */
class NegotiationUrlMatrixForm extends ConfigFormBase {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new NegotiationUrlForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager) {
    parent::__construct($config_factory);
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'language_negotiation_configure_url_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['language.negotiation'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    global $base_root;
    $config = $this->config('language.negotiation');
    $module_exists = [\Drupal::moduleHandler(), 'moduleExists'];

    $form['language'] = [
      '#type' => 'details',
      '#tree' => TRUE,
      '#title' => $this->t('Language Configuration'),
      '#open' => TRUE,
    ];

    if ($module_exists('key')) {
      $options['key'] = $this->t('Key');
    }
    $options['json'] = $this->t('JSON');

    $form['language']['language_negotiation_matrix'] = [
      '#title' => $this->t('Matrix Definition'),
      '#type' => 'select',
      '#description' => $this->t('This language negotiation configuration can accept two ways of implementing the matrix: via a KEY using the <strong>key</strong> module [%enabled] or through a JSON object.', ['%enabled' => $module_exists('key')?'Enabled':'Not Installed']),
      '#options' => $options,
      '#default_value' => $config->get('matrix.language.language_negotiation_matrix'),
    ];

    if ($module_exists('key')) {
      $form['language']['language_matrix'] = [
        '#type' => 'key_select',
        '#title' => $this->t('Language Matrix ENV variable'),
        '#description' => $this->t('To take advantage of this language detection you will require an ENVIRONMENT variable describing the language to path relationship. This will be consumed by the API and converted into a matrix for proper language switching. This functionality is born from a need to have the language and paths be supplied at runtime and likely highly customizable on a per project basis. This module will do its best to map the matrix in an intelligent way. It is possible to configure and export the results for more flexibility with Drupal.'),
        '#key_filters' => ['provider' => 'env', 'group' => 'generic'],
        '#states' => [
          'visible' => [
            ':input[name="language[language_negotiation_matrix]"]' => [
              'value' => (string) 'key',
            ],
          ],
        ],
      ];
    }

    $form['language']['language_matrix_json'] = [
      '#type' => 'codemirror',
      '#title' => $this->t('Language Matrix JSON'),
      '#description' => $this->t('Provide the matrix in JSON. To get the mappings to work, please SUBMIT the form and continue applying your settings.'),
      '#mode' => 'json',
      '#default_value' => $config->get('matrix.language.language_matrix_json'),
      '#states' => [
        'visible' => [
          ':input[name="language[language_negotiation_matrix]"]' => [
            'value' => (string) 'json',
          ],
        ],
      ],
    ];

    $form['language_negotiation_url_matrix_type'] = [
      '#title' => $this->t('Part of the URL that determines language'),
      '#type' => 'radios',
      '#options' => [
        LanguageNegotiationUrlMatrix::CONFIG_PATH_PREFIX => $this->t('Path prefix'),
        LanguageNegotiationUrlMatrix::CONFIG_DOMAIN => $this->t('Domain'),
      ],
      '#default_value' => $config->get('matrix.source'),
    ];

    $matrix = FALSE;
    if ($config->get('matrix.language.language_negotiation_matrix') == 'key') {
      if (!empty(\Drupal::service('key.repository')->getKey('language_matrix'))) {
        $matrix = \Drupal::service('key.repository')->getKey('language_matrix')->getKeyValues();
      }
    } else {
      $matrix = Json::decode($config->get('matrix.language.language_matrix_json'));
    }

    if (is_array($matrix)) {
      $default_value = $config->get('matrix.prefixes.mapping');
      $languages = $this->languageManager->getLanguages();
      $prefixes = $config->get('matrix.prefixes');
      $domains = $config->get('matrix.domains');

      foreach ($matrix as $langcode => $site_path) {
        $site_path = '/' . ltrim($site_path, '/');
        $requested_path[$langcode] = $site_path;
        if (isset($languages[$langcode])) {
          $language_mapping[$site_path] = $languages[$langcode]->getId();
        } else {
          $language_mapping[$site_path] = $langcode . ' (uninstalled)';
        }
      }

      $form['prefix'] = [
        '#type' => 'details',
        '#tree' => TRUE,
        '#title' => $this->t('Path prefix configuration'),
        '#open' => TRUE,
        '#states' => [
          'visible' => [
            ':input[name="language_negotiation_url_matrix_type"]' => [
              'value' => (string) LanguageNegotiationUrlMatrix::CONFIG_PATH_PREFIX,
            ],
          ],
        ],
      ];

      $form['prefix']['mapping'] = [
        '#type' => 'mapping',
        '#title' => $this->t('Language Matrix'),
        '#description_display' => 'before',
        '#source' => $requested_path,
        '#source__title' => $this->t('Path Requested'),
        '#destination' => $language_mapping,
        '#destination__title' => $this->t('Language Mapping'),
        '#destination__description' => NULL,
        '#default_value' => $default_value,
        '#filter' => FALSE,
        '#description' => $this->t('The Language Matrix Negotiation will attempt to map the paths to <strong>active</strong> languages per the environment variable provided. If you need to modify this list, please choose the desired mapping and save.'),
        '#prefix' => '<div id="mappings-wrapper">',
        '#suffix' => '</div>',
        '#required' => TRUE,
      ];

      $form['domain'] = [
        '#type' => 'details',
        '#tree' => TRUE,
        '#title' => $this->t('Domain configuration'),
        '#open' => TRUE,
        '#description' => $this->t('The domain names to use for these languages. <strong>Modifying this value may break existing URLs. Use with caution in a production environment.</strong> Example: Specifying "de.example.com" as language domain for German will result in a URL like "http://de.example.com/contact".'),
        '#states' => [
          'visible' => [
            ':input[name="language_negotiation_url_matrix_type"]' => [
              'value' => (string) LanguageNegotiationUrlMatrix::CONFIG_DOMAIN,
            ],
          ],
        ],
      ];

      $form['domain']['mapping'] = [
        '#type' => 'mapping',
        '#title' => $this->t('Language Matrix'),
        '#description_display' => 'before',
        '#source' => $requested_path,
        '#source__title' => $this->t('Path Requested'),
        '#destination' => $language_mapping,
        '#destination__title' => $this->t('Language Mapping'),
        '#destination__description' => NULL,
        '#default_value' => $default_value,
        '#filter' => FALSE,
        '#description' => $this->t('The Language Matrix Negotiation will attempt to map the paths to <strong>active</strong> languages per the environment variable provided. If you need to modify this list, please choose the desired mapping and save.'),
        '#prefix' => '<div id="mappings-wrapper">',
        '#suffix' => '</div>',
        '#required' => TRUE,
      ];

      $form['prefix']['set'] = [
        '#type' => 'markup',
        '#markup' => $this->t('<h4>Active Languages for Prefixes</h4>'),
      ];
      $form['domain']['set'] = [
        '#type' => 'markup',
        '#markup' => $this->t('<h4>Active Languages for Domains</h4>'),
      ];

      foreach ($languages as $langcode => $language) {
        $t_args = ['%language' => $language->getName(), '%langcode' => $language->getId()];
        $form['prefix'][$langcode] = [
          '#type' => 'textfield',
          '#title' => $language->isDefault() ? $this->t('%language (%langcode) path prefix (Default language)', $t_args) : $this->t('%language (%langcode) path prefix', $t_args),
          '#maxlength' => 64,
          '#default_value' => isset($prefixes[$langcode]) ? $prefixes[$langcode] : '',
          '#field_prefix' => $base_root . $requested_path[$langcode] .  '/',
        ];
        $form['domain'][$langcode] = [
          '#type' => 'textfield',
          '#title' => $this->t('%language (%langcode) domain', ['%language' => $language->getName(), '%langcode' => $language->getId()]),
          '#maxlength' => 128,
          '#default_value' => isset($domains[$langcode]) ? $domains[$langcode] : '',
          '#field_suffix' => $requested_path[$langcode] .  '/',
        ];
      }

    }

    $form_state->setRedirect('language.negotiation');

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $languages = $this->languageManager->getLanguages();

    // Count repeated values for uniqueness check.
    $types = $form_state->getValue('prefix');
    // remove "mapping" array
    unset($types['mapping']);
    $count = array_count_values($types);
    $default_langcode = $this->config('language.negotiation')->get('selected_langcode');
    if ($default_langcode == LanguageInterface::LANGCODE_SITE_DEFAULT) {
      $default_langcode = $this->languageManager->getDefaultLanguage()->getId();
    }
    foreach ($languages as $langcode => $language) {
      $value = $form_state->getValue(['prefix', $langcode]);
      if ($value === '') {
        if (!($default_langcode == $langcode) && $form_state->getValue('language_negotiation_url_matrix_type') == LanguageNegotiationUrlMatrix::CONFIG_PATH_PREFIX) {
          // Throw a form error if the prefix is blank for a non-default language,
          // although it is required for selected negotiation type.
          $form_state->setErrorByName("prefix][$langcode", $this->t('The prefix may only be left blank for the <a href=":url">selected detection fallback language.</a>', [
            ':url' => Url::fromRoute('language.negotiation_selected')->toString(),
          ]));
        }
      }
      elseif (strpos($value, '/') !== FALSE) {
        // Throw a form error if the string contains a slash,
        // which would not work.
        $form_state->setErrorByName("prefix][$langcode", $this->t('The prefix may not contain a slash.'));
      }
      elseif (isset($count[$value]) && $count[$value] > 1) {
        // Throw a form error if there are two languages with the same
        // domain/prefix.
        $form_state->setErrorByName("prefix][$langcode", $this->t('The prefix for %language, %value, is not unique.', ['%language' => $language->getName(), '%value' => $value]));
      }
    }

    // Count repeated values for uniqueness check.
    $types = $form_state->getValue('domain');
    // remove "mapping" array
    unset($types['mapping']);
    $count = array_count_values($types);
    foreach ($languages as $langcode => $language) {
      $value = $form_state->getValue(['domain', $langcode]);

      if ($value === '') {
        if ($form_state->getValue('language_negotiation_url_matrix_type') == LanguageNegotiationUrlMatrix::CONFIG_DOMAIN) {
          // Throw a form error if the domain is blank for a non-default language,
          // although it is required for selected negotiation type.
          $form_state->setErrorByName("domain][$langcode", $this->t('The domain may not be left blank for %language.', ['%language' => $language->getName()]));
        }
      }
      elseif (isset($count[$value]) && $count[$value] > 1) {
        // Throw a form error if there are two languages with the same
        // domain/domain.
        $form_state->setErrorByName("domain][$langcode", $this->t('The domain for %language, %value, is not unique.', ['%language' => $language->getName(), '%value' => $value]));
      }
    }

    // Domain names should not contain protocol and/or ports.
    foreach ($languages as $langcode => $language) {
      $value = $form_state->getValue(['domain', $langcode]);
      if (!empty($value)) {
        // Ensure we have exactly one protocol when checking the hostname.
        $host = 'http://' . str_replace(['http://', 'https://'], '', $value);
        if (parse_url($host, PHP_URL_HOST) != $value) {
          $form_state->setErrorByName("domain][$langcode", $this->t('The domain for %language may only contain the domain name, not a trailing slash, protocol and/or port.', ['%language' => $language->getName()]));
        }
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save selected format (prefix or domain).
    $this->config('language.negotiation')
      ->set('matrix.language', $form_state->getValue('language'))
      ->set('matrix.source', $form_state->getValue('language_negotiation_url_matrix_type'))
      // Save new domain and prefix values.
      ->set('matrix.prefixes', $form_state->getValue('prefix'))
      ->set('matrix.domains', $form_state->getValue('domain'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
